/*
   gpio.c
   David Rowe 30 Jan 2009
   Minimal OpenWRT/Atheros SoC device driver for testing GPIOs
*/

#include <linux/module.h>
#include <linux/kernel.h>
#include <ar531x.h>
#include <linux/proc_fs.h>

int proc_read_all(char *buf, char **start, off_t offset,
                  int count, int *eof, void *data)
{
    int len;

    len = sprintf(buf, 
		  "DI.: 0x%x\n"
		  "DO.: 0x%x\n"
		  "CR.: 0x%x\n"
		  "INT: 0x%x\n", 
		  (int)sysRegRead(AR5315_GPIO_DI), 
		  (int)sysRegRead(AR5315_GPIO_DO), 
		  (int)sysRegRead(AR5315_GPIO_CR), 
		  (int)sysRegRead(AR5315_GPIO_INT));

    *eof=1;
    return len;
}

int proc_read_do(char *buf, char **start, off_t offset,
                 int count, int *eof, void *data)
{
    int len;

    len = sprintf(buf,"DO.: 0x%x\n", (int)sysRegRead(AR5315_GPIO_DO));

    *eof=1;
    return len;
}

static int proc_write_do(struct file *file, const char *buffer,
			 unsigned long count, void *data)
{
    char *endbuffer;

    sysRegWrite(AR5315_GPIO_DO, simple_strtol (buffer, &endbuffer, 16));

    return count;
}

int proc_read_cr(char *buf, char **start, off_t offset,
                 int count, int *eof, void *data)
{
    int len;

    len = sprintf(buf,"CR.: 0x%x\n", (int)sysRegRead(AR5315_GPIO_CR));

    *eof=1;
    return len;
}

static int proc_write_cr(struct file *file, const char *buffer,
			 unsigned long count, void *data)
{
    char *endbuffer;

    sysRegWrite(AR5315_GPIO_CR, simple_strtol (buffer, &endbuffer, 16));
  
    return count;
}

static int __init gpio_init(void)
{
    struct proc_dir_entry *proc_do, *proc_cr;

    proc_mkdir("gpio", 0);
    create_proc_read_entry("gpio/all", 0, NULL, proc_read_all, NULL);
    proc_do = create_proc_read_entry("gpio/do", 0, NULL, proc_read_do, NULL);
    proc_cr = create_proc_read_entry("gpio/cr", 0, NULL, proc_read_cr, NULL);
    proc_do->write_proc = proc_write_do;
    proc_cr->write_proc = proc_write_cr;

    return 0;
}

static void __exit gpio_exit(void)
{
    remove_proc_entry("gpio", NULL);
}

module_init(gpio_init);
module_exit(gpio_exit);

