// tornado@odessaua.com tornado@dd-wrt.com
// **************************************************************************
//  Written by HairyDairyMaid (a.k.a. - lightbulb)
//  hairydairymaid@yahoo.com
// **************************************************************************
//
//  This program is copyright (C) 2004 HairyDairyMaid (a.k.a. Lightbulb)
//  This program is free software; you can redistribute it and/or modify it
//  under the terms of version 2 the GNU General Public License as published
//  by the Free Software Foundation.
//  This program is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
//  more details.
//  To view a copy of the license go to:
//  http://www.fsf.org/copyleft/gpl.html
//  To receive a copy of the GNU General Public License write the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// **************************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

#ifndef WINDOWS_VERSION

#include <unistd.h>
#include <sys/ioctl.h>

#ifdef __FreeBSD__
#include <dev/ppbus/ppi.h>
#include <dev/ppbus/ppbconf.h>
#define PPWDATA PPISDATA
#define PPRSTATUS PPIGSTATUS
#else
#include <linux/ppdev.h>
#endif

#endif

#define true  1
#define false 0

#define IXP425_266  0x19277013 // XScale IXP42X 266mhz
#define IXP425_400  0x19275013 // XScale IXP42X 400mhz
#define IXP425_533  0x19274013 // XScale IXP42X 533mhz
#define ARM_940T    0x10940027 // ARM 940T

#define RETRY_ATTEMPTS 16

// The ixdp425 has one 16-bit device.
// StrataFlash 28F128.

//#define CYGNUM_FLASH_DEVICES 	(1)
//#define CYGNUM_FLASH_BASE_MASK  (0xFF000000u) // 16Mb

//#define CYGNUM_FLASH_BASE 	(0x50000000u)
//#define CYGNUM_FLASH_WIDTH 	(16)
//#define CYGNUM_FLASH_BLANK      (1)

//#include <pkgconf/hal.h>  */ // for CYGHWR_HAL_ARM_BIGENDIAN

// We have to do some address gymnastics in little-endian mode
//#ifndef CYGHWR_HAL_ARM_BIGENDIAN
//#define __INV(a) ((flash_t *)((unsigned)(a) ^ 0x2))
//#define CYGHWR_FLASH_WRITE_BUF(a,b) (*__INV(a) = *__INV(b))
//#define CYGHWR_FLASH_READ_QUERY(a)  (*__INV(a))
//#endif

//#endif  // CYGONCE_DEVS_FLASH_IXDP425_STRATAFLASH_INL
// ------------------------------------------------------------------------
// EOF ixdp425_strataflash.inl

// IPX42XX provides flash uncached mapping at 0xB0000000 and a data coherent mapping at 0xA0000000.
#define	BYTE0(h)	((h) & 0xFF)
#define	BYTE1(h)	(((h) >> 8) & 0xFF)

#define SWAP_ONE(v) (((v & 0xff)<<8)    | ((v & 0xff00)>>8))

#define SWAP_TWO(v) (  ((v & 0xff)<<24)    | ((v & 0xff00)<<8)\
		       | ((v & 0xff0000)>>8) | ((v & 0xff000000)>>24))
/*
By rganapa@netravi.net

Mapping information for custom JTAG cables.

(nTRST, TDI, TMS, and TCK are connected to these Pins)

Pin 2 - 0
Pin 3 - 1
Pin 4 - 2
Pin 5 - 3
Pin 6 - 4
Pin 7 - 5
Pin 8 - 6
Pin 9 - 7

(TDO is connected to one of these Pins)

Pin 15 - 3
Pin 13 - 4
Pin 12 - 5
Pin 10 - 6
Pin 11 - 7

(Nothing is connected to these pins)

Pin 1  - 0
Pin 14 - 1
PIN 16 - 2
Pin 17 - 3

Pin 18-25 are connected to ground

Example:

let's say you've built a simple jtag cable like this

pin 2--->100 ohms ---> TDI
pin 3--->100 ohms ---> TCK
pin 4--->100 ohms ---> TMS
pin 5--->100 ohms ---> nTRST
pin 11--->100 ohms ---> TDO

Then using the above mapping

#define TDI	0 // pin 2 is mapped as 0
#define TCK	1 // pin 3 is mapped as 1
#define TMS	2 // pin 4 is mapped as 2
#define nTRST	3 // pin 5 is mapped as 3
#define TDO 	7 // pin 11 is mapped as 7

if you happen to have a buffered jtag like this 
(if you are into dishnet/FTA hacking then you have this jtag)

PIN 2--->74HC244 ---> TMS == mapped to 0
PIN 3--->74HC244 ---> TCK == mapped to 1
PIN 4--->74HC244 ---> TDI == mapped to 2
PIN 5--->74HC244 ---> nTRST == mapped to 3
PIN 13--->74HC244 ---> TDO == mapped to 4

Then using the above mapping

#define WTDI	2
#define WTCK	1
#define WTMS	0
#define WTRST_N	3
#define WTD0	4

How did I get this mapping?

Each lpt port is a 8bit register (D7-D0)
Printer         Data Port        Status          Control
LPT1            0x03bc          0x03bd          0x03be
LPT2            0x0378          0x0379          0x037a
LPT3            0x0278          0x0279          0x027a



To access individual bits the code is using C language's 'Bitwise Left Shift Operand <<" into 1

for example to individually address bit D7

data = 1 << 7 // gives you 10000000
data = 1 << 6 // gives you 01000000
data = 1 << 5 // gives you 00100000
.
.
.
data = 1 << 0 // gives you 00000001

Data port - 8 output lines      Status Port - 5 input lines	Control Port - 4 misc output lines

D7 - Data 7                     D7 - Busy			D7 - Reserved			
D6 - Data 6                     D6 - ^ACK			D6 - Reserved			
D5 - Data 6                     D5 - PE				D5 - Direction
D4 - Data 4                     D4 - Select			D4 - IRQ_Enable
D3 - Data 3                     D3 - ^ERROR			D3 - ^Select_In
D2 - Data 2                     D2 - ^IRQ			D2 - INIT
D1 - Data 1                     D1 - Reserved			D1 - ^Autofeed
D0 - Data 0                     D0 - Reserved			D0 - ^Strobe

You can search the internet for LPT pin description assignment.

Perhaps some one can externlize the lpt pin mapping in a config file, it saves compile headaches and can use it with any JTAG. Same applies for the cpu/flash chips as well. Jkeys from dishnet does a very good job at this. I am including this information hoping someone will improve this code further - Ravi
*/

// --- Xilinx Type Cable ---

#define TDI	0 // pin 2 is mapped as 0
#define TCK	1 // pin 3 is mapped as 1
#define TMS	2 // pin 4 is mapped as 2
#define nTRST	3 // pin 5 is mapped as 3
#define TDO 	7 // pin 11 is mapped as 7

//#define TDI     0
//#define TCK     1
//#define TMS     2
//#define TDO     4	 // this is from the status pin
//#define TRST    4	 // this is from the data pin



// ---Wiggler Type Cable pinout modified for Alan's Wiggler compatible schematic from the Openwrt page---


//#define WSRST_N   0
#define WTMS      1
#define WTCK      2
#define WTDI      3
#define WTRST_N   4
#define WTDO      7


// SPI Flash commands
#define SPI_WREN	(0x06)  // Set Write Enable Latch
#define SPI_WRDI	(0x04)  // Reset Write Enable Latch
#define SPI_RDSR	(0x05)  // Read Status Register
#define SPI_WRSR	(0x01)  // Write Status Register
#define SPI_READ	(0x03)  // Read data from memory
#define SPI_PP		(0x02)  // Program Data into memory
#define SPI_SE		(0xD8)  // Erase one sector in memory
#define SPI_BE		(0xC7)  // Erase all memory
#define SPI_RES		(0xAB)  // Read Electonic Signature
#define SPI_RDID	(0x9f)  // Read Identification
#define WIP		(0x1)	// Check the write in progress bit of the SPI status register
#define WEL		(0x2)	// Check the write enable bit of the SPI status register
#define TIMEOUT		500

// --- Some EJTAG Instruction Registers ---
// 0x7e idcode ipx425
#define INSTR_EXTEST    0x00
#define INSTR_IDCODE    0x01
#define INSTR_SAMPLE    0x02
#define INSTR_IMPCODE   0x03
#define INSTR_ADDRESS   0x08
#define INSTR_DATA      0x09
#define INSTR_CONTROL   0x0A
#define INSTR_ALL       0x0B
#define INSTR_EJTAGBOOT 0x0C
#define INSTR_RESET     0x0D     // makes the processor execute the reset handler after reset
#define INSTR_EJWATCH   0x1C
#define INSTR_BYPASS    0xFF


/*
#define INSTR_EXTEST    0x00
#define INSTR_IDCODE    0xFE
#define INSTR_SAMPLE    0x01
#define INSTR_BYPASS    0xFF

*/
/*
#define INSTR_EXTEST          0x0 // 0000
//#define INSTR_SCAN            0x2 // 0010
//#define INSTR_INTEST          0xC // 1100
#define INSTR_IDCODE          0xE // 1110
#define INSTR_BYPASS          0xF // 1111
//#define INSTR_CLAMP           0x5 // 0101
//#define INSTR_HIGHZ           0x7 // 0111
//#define INSTR_CLAMPZ          0x9 // 1001
//#define INSTR_SAMPLE          0x3 // 0011
//#define INSTR_RESTART         0x4 // 0100
*/

// --- Some EJTAG Bit Masks ---
#define TOF             (1 << 1 )
#define BRKST           (1 << 3 )
#define DRWN            (1 << 9 )
#define DERR            (1 << 10)
#define DSTRT           (1 << 11)
#define SETDEV          (1 << 14)
#define PROBEN          (1 << 15)
#define DMAACC          (1 << 17)
#define PRACC           (1 << 18)
#define PRNW            (1 << 19)
#define DLOCK           (1 << 5 )
#define TIF             (1 << 2 )
#define SYNC            (1 << 23)
#define PRRST           (1 << 16)
#define PERRST          (1 << 20)
#define JTAGBRK         (1 << 12)
#define DNM             (1 << 28)
#define ROCC            (1 << 31)

#define DMA_BYTE        0x00000000  //DMA tranfser size BYTE
#define DMA_HALFWORD    0x00000080  //DMA transfer size HALFWORD
#define DMA_WORD        0x00000100  //DMA transfer size WORD
#define DMA_TRIPLEBYTE  0x00000180  //DMA transfer size TRIPLEBYTE

#define _4K         0x00001000
#define _8K         0x00002000
#define _16K        0x00004000
#define _32K        0x00008000
#define _64K        0x00010000
#define _128K       0x00020000
#define _256K       0x00040000
#define _512K       0x00080000

#define _1M         0x00100000
#define _2M         0x00200000
#define _4M         0x00400000
#define _8M         0x00800000
#define _16M        0x01000000
#define _32M        0x02000000
#define _64M        0x04000000
#define _128M       0x08000000
#define _256M       0x10000000
#define _512M       0x20000000

#define  size4K        0x1000
#define  size8K        0x2000
#define  size16K       0x4000
#define  size32K       0x8000
#define  size64K       0x10000
#define  size128K      0x20000

#define  size1MB       0x100000
#define  size2MB       0x200000
#define  size4MB       0x400000
#define  size8MB       0x800000
#define  size16MB      0x1000000

#define  CMD_TYPE_BSC  0x01
#define  CMD_TYPE_SCS  0x02
#define  CMD_TYPE_AMD  0x03
#define  CMD_TYPE_SST  0x04
#define  CMD_TYPE_SPI  0x05

#define  STATUS_READY  0x0080


// EJTAG DEBUG Unit Vector on Debug Break
#define MIPS_DEBUG_VECTOR_ADDRESS           0xFF200200

// Our 'Pseudo' Virtual Memory Access Registers
#define MIPS_VIRTUAL_ADDRESS_ACCESS         0xFF200000
#define MIPS_VIRTUAL_DATA_ACCESS            0xFF200004


// --- Uhh, Just Because I Have To ---
void chip_detect(void);
void chip_shutdown(void);
static unsigned char clockin(int tms, int tdi);
void define_block(unsigned int block_count, unsigned int block_size);
static unsigned int ejtag_read(unsigned int addr);
static unsigned int ejtag_read_h(unsigned int addr);
void ejtag_write(unsigned int addr, unsigned int data);
void ejtag_write_h(unsigned int addr, unsigned int data);
static unsigned int ejtag_dma_read(unsigned int addr);
static unsigned int ejtag_dma_read_h(unsigned int addr);
void ejtag_dma_write(unsigned int addr, unsigned int data);
void ejtag_dma_write_h(unsigned int addr, unsigned int data);
static unsigned int ejtag_pracc_read(unsigned int addr);
void ejtag_pracc_write(unsigned int addr, unsigned int data);
static unsigned int ejtag_pracc_read_h(unsigned int addr);
void ejtag_pracc_write_h(unsigned int addr, unsigned int data);
void identify_flash_part(void);
void lpt_closeport(void);
void lpt_openport(void);
static unsigned int ReadData(void);
static unsigned int ReadWriteData(unsigned int in_data);
void run_backup(char *filename, unsigned int start, unsigned int length);
void run_erase(char *filename, unsigned int start, unsigned int length);
void run_flash(char *filename, unsigned int start, unsigned int length);
void set_instr(int instr);
void sflash_config(void);
void sflash_erase_area(unsigned int start, unsigned int length);
void sflash_erase_block(unsigned int addr);
void sflash_probe(void);
void sflash_reset(void);
void sflash_write_word(unsigned int addr, unsigned int data);
void show_usage(void);
void ShowData(unsigned int value);
void test_reset(void);
void WriteData(unsigned int in_data);
void ExecuteDebugModule(unsigned int *pmodule);
void check_ejtag_features(void);
void unlock_bypass(void);
void unlock_bypass_reset(void);

unsigned int pracc_readword_code_module[] =
{
  // #
  // # HairyDairyMaid's Assembler PrAcc Read Word Routine
  // #
  // start:
  //
  // # Load R1 with the address of the pseudo-address register
  0x3C01FF20,  // lui $1,  0xFF20
  0x34210000,  // ori $1,  0x0000
  //
  // # Load R2 with the address for the read
  0x8C220000,  // lw $2,  ($1)
  //
  // # Load R3 with the word @R2
  0x8C430000,  // lw $3, 0($2)
  //
  // # Store the value into the pseudo-data register
  0xAC230004,  // sw $3, 4($1)
  //
  0x00000000,  // nop
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
}; // nop


unsigned int pracc_writeword_code_module[] =
{
  // #
  // # HairyDairyMaid's Assembler PrAcc Write Word Routine
  // #
  // start:
  //
  // # Load R1 with the address of the pseudo-address register
  0x3C01FF20,  // lui $1,  0xFF20
  0x34210000,  // ori $1,  0x0000
  //
  // # Load R2 with the address for the write
  0x8C220000,  // lw $2,  ($1)
  //
  // # Load R3 with the data from pseudo-data register
  0x8C230004,  // lw $3, 4($1)
  //
  // # Store the word at @R2 (the address)
  0xAC430000,  // sw $3,  ($2)
  //
  0x00000000,  // nop
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
}; // nop


unsigned int pracc_readhalf_code_module[] =
{
  // #
  // # HairyDairyMaid's Assembler PrAcc Read HalfWord Routine
  // #
  // start:
  //
  // # Load R1 with the address of the pseudo-address register
  0x3C01FF20,  // lui $1,  0xFF20
  0x34210000,  // ori $1,  0x0000
  //
  // # Load R2 with the address for the read
  0x8C220000,  // lw $2,  ($1)
  //
  // # Load R3 with the half word @R2
  0x94430000,  // lhu $3, 0($2)
  //
  // # Store the value into the pseudo-data register
  0xAC230004,  // sw $3, 4($1)
  //
  0x00000000,  // nop
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
}; // nop


unsigned int pracc_writehalf_code_module[] =
{
  // #
  // # HairyDairyMaid's Assembler PrAcc Write HalfWord Routine
  // #
  // start:
  //
  // # Load R1 with the address of the pseudo-address register
  0x3C01FF20,  // lui $1,  0xFF20
  0x34210000,  // ori $1,  0x0000
  //
  // # Load R2 with the address for the write
  0x8C220000,  // lw $2,  ($1)
  //
  // # Load R3 with the data from pseudo-data register
  0x8C230004,  // lw $3, 4($1)
  //
  // # Store the half word at @R2 (the address)
  0xA4430000,  // sh $3,  ($2)
  //
  0x00000000,  // nop
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
}; // nop

unsigned int pracc_return_from_debug[] =
{
  // start:
  //
  // # Load R1 with the address of the pseudo-address register
//  0x3C010000,  // lui   $1,   0x8000
//  0x34210000,  // ori   $1,   0x0000   # $1 <-  ($1 | 0x0000)
  0x00000000,  // nop
  0x00000000,  // nop
  //
  // # Load R1 to R24(DEPC) of CP0 
//  0x4101C000,  // mtc0 $24,   0, 0     # DEPC <- $1
  0x00000000,  // nop
  0x00000000,  // nop

  0x4200001F,  // deret
  0x00000000,  // nop 
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
};



unsigned int pracc_read_depc[] =
{
  // #
  // # HairyDairyMaid's Assembler PrAcc Read Word Routine
  // #
  // start:
  //
  // # Load R1 with the address of the pseudo-address register
  0x3C01FF20,  // lui $1,  0xFF20
  0x34210000,  // ori $1,  0x0000
  //
  // # Load R2 with the DEPC
  0x4002C000,  // mfc0 $2, $24(DEPC)   (fetch R2 <- DEFC to R2)
  //
  // # nop
  0x00000000,  // 
  //
  // # Store the R2 value into the pseudo-data register
  0xAC220004,  // sw $2, 4($1)
  //
  0x00000000,  // nop
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
};

unsigned int pracc_write_depc[] =
{
  // # Load R1 with the address of the pseudo-address register
  0x3C01FF20,  // lui $1,  0xFF20
  0x34210000,  // ori $1,  0x0000
  //
  // # Load R2 with the data from pseudo-data register
  0x8C220004,  // lw $3, 4($1)
  //0x3C02F800,  // lui $2,  0x8000
  //0x34420000,  // ori $2,  0x0000
  //
  // # store DEPC from R2
  0x4082C000,  // mtc0 $24(DEPC),   $2, 0     # DEPC <- $2
  0x00000000,  // nop
  0x00000000,  // nop (put some delay for DEPC update to take place)
  //
  0x1000FFF9,  // beq $0, $0, start
  0x00000000
};
// **************************************************************************
// End of File
// **************************************************************************
